//---------------------------------------------------------------------------------------
//  FILE:    X2Ability_SharpshooterAbilitySet.uc
//  AUTHOR:  Edward Sim
//  DATE:    2016-02-09
//  PURPOSE: Defines abilities used by the Officer class.
//---------------------------------------------------------------------------------------
class X2Ability_OfficerAbilitySet extends X2Ability
	config(GameData_SoldierSkills);
	
var config int MARK_TARGET_LOCAL_COOLDOWN;
var config name MARK_TARGET_ATTACHTOSOCKETNAME;
var config name MARK_TARGET_ATTACHTOSOCKETSARRAYNAME;

var config int FLARE_COOLDOWN;
var config int FLARE_GLOBAL_COOLDOWN;
var config int FLARE_CHARGES;
var config int FLARE_DURATION;
var config int FLARE_RADIUS_METERS;

var config int COVER_COOLDOWN;
var config int COVER_GLOBAL_COOLDOWN;
var config int COVER_DEFENSE;

var config float ENERGY_SHIELD_RANGE_METERS;
var config int ENERGY_SHIELD_COOLDOWN;
var config int ENERGY_SHIELD_GLOBAL_COOLDOWN;
var config int ENERGY_SHIELD_DURATION;
var config int ENERGY_SHIELD_HP;

var config name ENERGY_SHIELD_ATTACHTOSOCKETNAME;
var config name ENERGY_SHIELD_ATTACHTOSOCKETSARRAYNAME;


var config int SECRET_MISSION_COOLDOWN;
var config int SECRET_MISSION_GLOBAL_COOLDOWN;
var config int SECRET_MISSION_CHARGES;
var config int SECRET_MISSION_AIM;
var config int SECRET_MISSION_DODGE;
var config int SECRET_MISSION_DAMAGE;

var config int STEALTH_CHARGES;

var config int AIR_RAID_LOCAL_COOLDOWN;
var config int AIR_RAID_GLOBAL_COOLDOWN;
var config int AIR_RAID_TARGETING_AREA_RADIUS;
var config float AIR_RAID_IMPACT_RADIUS_METERS;
var config int AIR_RAID_DAMAGE_AMOUNT;
var config WeaponDamageValue AIR_RAID_WEAPON_DAMAGE;

var config int ALERT_COMMAND_SIGHT;

var config int ENCOURAGE_COMMAND_BONUS_WILL;

var config int DEFENSE_COMMAND_CHANCE;
var config int DEFENSE_COMMAND_AMOUNT;

var config int BLITZ_COMMAND_MOVE;

var config int TACTICAL_COMMAND_AIM;
var config int TACTICAL_COMMAND_DODGE;

var config int FIRE_COMMAND_CIRT;

var localized string TargetGettingMarkedFriendlyName;
var localized string TargetGettingMarkedFriendlyDesc;
	
var name AirRaidEffectName;
var name AirRaidTriggerName;

static function array<X2DataTemplate> CreateTemplates()
{
	local array<X2DataTemplate> Templates;
	
	Templates.AddItem(CreateMarkTargetAbility()); // ok
	
	Templates.AddItem(CreateFlareAbility()); // ok
	Templates.AddItem(CoverAbility());
	Templates.AddItem(CreateEnergyShieldAbility()); // ok
	Templates.AddItem(SecretMissionAbility());
	Templates.AddItem(SquadStealthAbility()); // ok
	Templates.AddItem(AirRaidRequestAbility());
	Templates.AddItem(AirRaidAttackAbility());

	
	Templates.AddItem(AlertCommandAbility()); // ok
	Templates.AddItem(EncourageCommandAbility()); // ok
	Templates.AddItem(DefenseCommandAbility()); // ok
	Templates.AddItem(BlitzCommandAbility()); // ok
	Templates.AddItem(FireCommandAbility()); // ok
	Templates.AddItem(TacticalCommandAbility()); // ok

	return Templates;
}

static function X2AbilityTemplate CreateMarkTargetAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityCost_ActionPoints ActionPointCost;
	local X2Condition_UnitProperty UnitPropertyCondition;
	local X2Condition_Visibility TargetVisibilityCondition;
	local X2Condition_UnitEffects UnitEffectsCondition;
	local X2AbilityTarget_Single SingleTarget;
	local X2AbilityTrigger_PlayerInput InputTrigger;
	local X2Effect_Persistent MarkedEffect;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_MarkTarget');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_advent_marktarget";
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.Hostility = eHostility_Offensive;

	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.MARK_TARGET_LOCAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bFreeCost = false;
	Template.AbilityCosts.AddItem(ActionPointCost);

	// The shooter cannot be mind controlled
	UnitEffectsCondition = new class'X2Condition_UnitEffects';
	UnitEffectsCondition.AddExcludeEffect(class'X2Effect_MindControl'.default.EffectName, 'AA_UnitIsMindControlled');
	Template.AbilityShooterConditions.AddItem(UnitEffectsCondition);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);

	// Target must be an enemy
	UnitPropertyCondition = new class'X2Condition_UnitProperty';
	UnitPropertyCondition.ExcludeDead = true;
	UnitPropertyCondition.ExcludeFriendlyToSource = true;
	UnitPropertyCondition.RequireWithinRange = false;
	Template.AbilityTargetConditions.AddItem(UnitPropertyCondition);
	
	// Target must be visible and may use squad sight
	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);

	// Target cannot already be marked
	UnitEffectsCondition = new class'X2Condition_UnitEffects';
	UnitEffectsCondition.AddExcludeEffect(class'X2StatusEffects'.default.MarkedName, 'AA_UnitIsMarked');
	Template.AbilityTargetConditions.AddItem(UnitEffectsCondition);

	// 100% chance to hit
	Template.AbilityToHitCalc = default.DeadEye;

	SingleTarget = new class'X2AbilityTarget_Single';
	Template.AbilityTargetStyle = SingleTarget;

	InputTrigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(InputTrigger);

	// Create the Marked effect
	MarkedEffect = class'X2StatusEffects'.static.CreateMarkedEffect(2, false);
	MarkedEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	Template.AddTargetEffect(MarkedEffect); //BMU - changing to an immediate execution for evaluation

	Template.CustomFireAnim = 'HL_SignalPoint';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	Template.BuildVisualizationFn = TargetGettingMarked_BuildVisualization;
	Template.CinescriptCameraType = "Mark_Target";
	
	return Template;
}

static function MarkedVisualizationRemoved(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameState_Unit UnitState;
	local X2Action_PlayEffect				EffectAction;

	UnitState = XComGameState_Unit(BuildTrack.StateObject_NewState);
	if (UnitState == none)
		return;

	EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, VisualizeGameState.GetContext()));
	EffectAction.AttachToUnit = true;
	EffectAction.EffectName = "FX_Holo_Targeting.P_Tracer_Beam_Target_Red";
	EffectAction.AttachToSocketName = default.MARK_TARGET_ATTACHTOSOCKETNAME;
	EffectAction.AttachToSocketsArrayName = default.MARK_TARGET_ATTACHTOSOCKETSARRAYNAME;
	EffectAction.bStopEffect = true;

	// dead units should not be reported
	if( !UnitState.IsAlive() )
	{
		return;
	}

	class'X2StatusEffects'.static.AddEffectMessageToTrack(BuildTrack, class'X2StatusEffects'.default.MarkedEffectLostString, VisualizeGameState.GetContext());
	class'X2StatusEffects'.static.UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}

simulated function TargetGettingMarked_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory				History;
	local XComGameStateContext_Ability		Context;
	local StateObjectReference				ShooterUnitRef;
	local StateObjectReference				TargetUnitRef;
	local XComGameState_Ability				Ability;
	local X2AbilityTemplate					AbilityTemplate;
	local AbilityInputContext				AbilityContext;
	
	local VisualizationTrack				EmptyTrack;
	local VisualizationTrack				BuildTrack;

	local X2Action_PlayEffect				EffectAction;
	local X2Action_PlaySoundAndFlyOver		SoundAndFlyOver;
	local X2Action_WaitForAbilityEffect		WaitForAbilityEffect;
	local X2Action_SendInterTrackMessage    SendMessageAction;
	local X2Action_PlayAnimation            PlayAnimation;

	local Actor								TargetVisualizer, ShooterVisualizer;
	local X2VisualizerInterface				TargetVisualizerInterface, ShooterVisualizerInterface;
	local int								EffectIndex;

	local name								ApplyResult;
	

	History = `XCOMHISTORY;
	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());

	AbilityContext = Context.InputContext;
	Ability = XComGameState_Ability(History.GetGameStateForObjectID(AbilityContext.AbilityRef.ObjectID));
	AbilityTemplate = class'XComGameState_Ability'.static.GetMyTemplateManager().FindAbilityTemplate(AbilityContext.AbilityTemplateName);
	
	//Configure the visualization track for the shooter
	//****************************************************************************************
	ShooterUnitRef = Context.InputContext.SourceObject;
	ShooterVisualizer = History.GetVisualizer(ShooterUnitRef.ObjectID);
	ShooterVisualizerInterface = X2VisualizerInterface(ShooterVisualizer);

	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(ShooterUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(ShooterUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(ShooterUnitRef.ObjectID);

	if (AbilityTemplate != None)
	{
		if (!AbilityTemplate.bSkipFireAction && !AbilityTemplate.bSkipExitCoverWhenFiring)
		{
			class'X2Action_ExitCover'.static.AddToVisualizationTrack(BuildTrack, Context);
		}
	}

	SoundAndFlyOver = X2Action_PlaySoundAndFlyOver(class'X2Action_PlaySoundAndFlyOver'.static.AddToVisualizationTrack(BuildTrack, Context));
	SoundAndFlyOver.SetSoundAndFlyOverParameters(None, Ability.GetMyTemplate().LocFlyOverText, '', eColor_Good);

	// Send an intertrack message letting the target know it can now die
	SendMessageAction = X2Action_SendInterTrackMessage(class'X2Action_SendInterTrackMessage'.static.AddToVisualizationTrack(BuildTrack, Context));
	SendMessageAction.SendTrackMessageToRef = Context.InputContext.PrimaryTarget;

	PlayAnimation = X2Action_PlayAnimation(class'X2Action_PlayAnimation'.static.AddToVisualizationTrack(BuildTrack, Context));
	PlayAnimation.Params.AnimName = 'HL_SignalPointA';

	if (AbilityTemplate != None && AbilityTemplate.AbilityTargetEffects.Length > 0)			//There are effects to apply
	{
		//Add any X2Actions that are specific to this effect being applied. These actions would typically be instantaneous, showing UI world messages
		//playing any effect specific audio, starting effect specific effects, etc. However, they can also potentially perform animations on the 
		//track actor, so the design of effect actions must consider how they will look/play in sequence with other effects.
		for (EffectIndex = 0; EffectIndex < AbilityTemplate.AbilityTargetEffects.Length; ++EffectIndex)
		{
			ApplyResult = Context.FindTargetEffectApplyResult(AbilityTemplate.AbilityTargetEffects[EffectIndex]);

			// Source effect visualization
			AbilityTemplate.AbilityTargetEffects[EffectIndex].AddX2ActionsForVisualizationSource(VisualizeGameState, BuildTrack, ApplyResult);
		}
	}

	if (ShooterVisualizerInterface != none)
	{
		ShooterVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
	}

	if (AbilityTemplate != None)
	{
		if (!AbilityTemplate.bSkipFireAction && !AbilityTemplate.bSkipExitCoverWhenFiring)
		{
			class'X2Action_EnterCover'.static.AddToVisualizationTrack(BuildTrack, Context);
		}
	}

	OutVisualizationTracks.AddItem(BuildTrack);
	//****************************************************************************************

	//Configure the visualization track for the target
	//****************************************************************************************
	TargetUnitRef = Context.InputContext.PrimaryTarget;
	TargetVisualizer = History.GetVisualizer(AbilityContext.PrimaryTarget.ObjectID);
	TargetVisualizerInterface = X2VisualizerInterface(TargetVisualizer);

	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(TargetUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(TargetUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(TargetUnitRef.ObjectID);
					
	Ability = XComGameState_Ability(History.GetGameStateForObjectID(Context.InputContext.AbilityRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1));

	WaitForAbilityEffect = X2Action_WaitForAbilityEffect(class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context));
	WaitForAbilityEffect.ChangeTimeoutLength(5.0f);

	EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, Context));
	EffectAction.AttachToUnit = true;
	EffectAction.EffectName = "FX_Holo_Targeting.P_Tracer_Beam_Target_Red";
	EffectAction.AttachToSocketName = default.MARK_TARGET_ATTACHTOSOCKETNAME;
	EffectAction.AttachToSocketsArrayName = default.MARK_TARGET_ATTACHTOSOCKETSARRAYNAME;
	
	if (AbilityTemplate != None && AbilityTemplate.AbilityTargetEffects.Length > 0)			//There are effects to apply
	{
		//Add any X2Actions that are specific to this effect being applied. These actions would typically be instantaneous, showing UI world messages
		//playing any effect specific audio, starting effect specific effects, etc. However, they can also potentially perform animations on the 
		//track actor, so the design of effect actions must consider how they will look/play in sequence with other effects.
		for (EffectIndex = 0; EffectIndex < AbilityTemplate.AbilityTargetEffects.Length; ++EffectIndex)
		{
			ApplyResult = Context.FindTargetEffectApplyResult(AbilityTemplate.AbilityTargetEffects[EffectIndex]);

			// Target effect visualization
			AbilityTemplate.AbilityTargetEffects[EffectIndex].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, ApplyResult);
		}

		if (TargetVisualizerInterface != none)
		{
			//Allow the visualizer to do any custom processing based on the new game state. For example, units will create a death action when they reach 0 HP.
			TargetVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
		}
	}
	
	OutVisualizationTracks.AddItem(BuildTrack);
	//****************************************************************************************
}

static function X2AbilityTemplate CreateFlareAbility()
{	
	local X2AbilityTemplate Template;
	local X2AbilityCost_ActionPoints ActionPointCost;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;
	local X2AbilityCharges                      Charges;
	local X2AbilityMultiTarget_Radius MultiTarget;
	local X2Effect_MarkValidActivationTiles MarkTilesEffect;
	local X2AbilityTarget_Cursor CursorTarget;
	local X2Effect_ScanningProtocol     ScanningEffect;
	local X2Condition_UnitProperty      CivilianProperty;
	local X2Effect_PersistentSquadViewer    ViewerEffect;
	local X2Effect_Persistent PersistentEffect;
	
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_Flare');
	
	Template.IconImage =  "img:///UILibrary_PerkIcons.UIPerk_grenade_flash";

	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.bShowActivation = true;
	Template.Hostility = eHostility_Neutral;

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = false;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.FLARE_COOLDOWN;
	Cooldown.NumGlobalTurns = default.FLARE_GLOBAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = default.FLARE_CHARGES;
	Template.AbilityCharges = Charges;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	MultiTarget = new class 'X2AbilityMultiTarget_Radius';
	MultiTarget.fTargetRadius = default.FLARE_RADIUS_METERS;
	MultiTarget.bIgnoreBlockingCover = true;
	Template.AbilityMultiTargetStyle = MultiTarget;
	
	MarkTilesEffect = new class'X2Effect_MarkValidActivationTiles';
	MarkTilesEffect.AbilityToMark = 'Officer_Flare';
	MarkTilesEffect.OnlyUseTargetLocation = true;
	Template.AddShooterEffect(MarkTilesEffect);

	CursorTarget = new class'X2AbilityTarget_Cursor';
	CursorTarget.bRestrictToSquadsightRange = true;
	CursorTarget.FixedAbilityRange = default.FLARE_DURATION;
	Template.AbilityTargetStyle = CursorTarget;
	
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
			
	Template.TargetingMethod = class'X2TargetingMethod_Flare';

	ScanningEffect = new class'X2Effect_ScanningProtocol';
	ScanningEffect.BuildPersistentEffect(10, true, false, false, eGameRule_PlayerTurnEnd);
	ScanningEffect.TargetConditions.AddItem(default.LivingHostileUnitOnlyProperty);
	Template.AddMultiTargetEffect(ScanningEffect);

	ScanningEffect = new class'X2Effect_ScanningProtocol';
	ScanningEffect.BuildPersistentEffect(10, true, false, false, eGameRule_PlayerTurnEnd);
	CivilianProperty = new class'X2Condition_UnitProperty';
	CivilianProperty.ExcludeNonCivilian = true;
	CivilianProperty.ExcludeHostileToSource = false;
	CivilianProperty.ExcludeFriendlyToSource = false;
	ScanningEffect.TargetConditions.AddItem(CivilianProperty);
	ScanningEffect.EffectRemovedVisualizationFn = FlareVisualizationRemoved;
	Template.AddMultiTargetEffect(ScanningEffect);
	
	ViewerEffect = new class'X2Effect_PersistentSquadViewer';
	ViewerEffect.BuildPersistentEffect(10, true, false, false, eGameRule_PlayerTurnBegin);
	ViewerEffect.bUseWeaponRadius = false;
	ViewerEffect.ViewRadius = default.FLARE_RADIUS_METERS;
	ViewerEffect.bUseSourceLocation = false;
	ViewerEffect.EffectRemovedVisualizationFn = FlareVisualizationRemoved;
	Template.AddShooterEffect(ViewerEffect);

	PersistentEffect = new class 'X2Effect_Persistent';
	PersistentEffect.BuildPersistentEffect(10, false, false,,eGameRule_PlayerTurnEnd);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, false, "", Template.AbilitySourceName);
	//PersistentEffect.SetDisplayInfo(ePerkBuff_Passive,Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,false,Template.AbilitySourceName);
	PersistentEffect.EffectRemovedVisualizationFn = FlareVisualizationRemoved;
	Template.AddMultiTargetEffect(PersistentEffect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	Template.BuildVisualizationFn = Flare_BuildVisualization;

	return Template;
}

simulated function Flare_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability Context;
	local StateObjectReference InteractingUnitRef;
	local X2VisualizerInterface Visualizer;
	local VisualizationTrack FlareBuildTrack;
	local X2Action_PlayEffect EffectAction;
	local XComGameState_Unit FlareUnit;
	local XComWorldData World;
	local vector TargetLocation;
	local TTile TargetTile;
	local X2Action_TimedWait WaitAction;	
	local X2Action_RevealArea			RevealAreaAction;
	local X2Action_PlayAnimation PlayAnimation;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());

	InteractingUnitRef = Context.InputContext.SourceObject;
	FlareBuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	FlareBuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	FlareBuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	FlareUnit = XComGameState_Unit(FlareBuildTrack.StateObject_NewState);

	if( FlareUnit != none )
	{
		World = `XWORLD;

		// Exit cover
		class'X2Action_ExitCover'.static.AddToVisualizationTrack(FlareBuildTrack, Context);

		class'X2Action_Fire_OpenUnfinishedAnim'.static.AddToVisualizationTrack(FlareBuildTrack, Context);

		PlayAnimation = X2Action_PlayAnimation(class'X2Action_PlayAnimation'.static.AddToVisualizationTrack(FlareBuildTrack, Context));
		PlayAnimation.Params.AnimName = 'HL_SignalPointA';

		// Wait to time the start of the warning FX
		WaitAction = X2Action_TimedWait(class'X2Action_TimedWait'.static.AddToVisualizationTrack(FlareBuildTrack, Context));
		WaitAction.DelayTimeSec = 0.5;

		TargetLocation = Context.InputContext.TargetLocations[0];
		TargetTile = World.GetTileCoordinatesFromPosition(TargetLocation);
		
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(FlareBuildTrack, Context));
		EffectAction.EffectName = "FX_Evac_Zone.P_EZ_Flare_and_Trail_Universal";
		EffectAction.EffectLocation = World.GetPositionFromTileCoordinates(TargetTile);
		
		RevealAreaAction = X2Action_RevealArea(class'X2Action_RevealArea'.static.AddToVisualizationTrack(FlareBuildTrack, Context));
		RevealAreaAction.TargetLocation = TargetLocation;
		RevealAreaAction.AssociatedObjectID = InteractingUnitRef.ObjectID;
		RevealAreaAction.ScanningRadius = class'XComWorldData'.const.WORLD_StepSize * default.FLARE_RADIUS_METERS;
		RevealAreaAction.bDestroyViewer = false;

		Visualizer = X2VisualizerInterface(FlareBuildTrack.TrackActor);
		if( Visualizer != none )
		{
			Visualizer.BuildAbilityEffectsVisualization(VisualizeGameState, FlareBuildTrack);
		}

		class'X2Action_EnterCover'.static.AddToVisualizationTrack(FlareBuildTrack, Context);

		OutVisualizationTracks.AddItem(FlareBuildTrack);
	}
}

static function FlareVisualizationRemoved(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameState_Unit UnitState;
	local X2Action_PlayEffect				EffectAction;
	local X2Action_RevealArea			RevealAreaAction;
	//local XComWorldData World;
	//local vector TargetLocation;
	//local TTile TargetTile;


	//UnitState = XComGameState_Unit(BuildTrack.StateObject_NewState);
	//if (UnitState == none)
	//	return;
	
	//World = `XWORLD;

	//TargetLocation = XComGameStateContext_Ability(VisualizeGameState.GetContext()).InputContext.TargetLocations[0];
	//TargetTile = World.GetTileCoordinatesFromPosition(TargetLocation);

	EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, VisualizeGameState.GetContext()));
	EffectAction.EffectName = "FX_Evac_Zone.P_EZ_Flare_and_Trail_Universal";
	//EffectAction.EffectLocation = World.GetPositionFromTileCoordinates(TargetTile);
	EffectAction.bStopEffect = true;
	
	RevealAreaAction = X2Action_RevealArea(class'X2Action_RevealArea'.static.AddToVisualizationTrack(BuildTrack, VisualizeGameState.GetContext()));
	RevealAreaAction.AssociatedObjectID = XComGameStateContext_Ability(VisualizeGameState.GetContext()).InputContext.SourceObject.ObjectID;
	//RevealAreaAction.TargetLocation = TargetLocation;
	RevealAreaAction.bDestroyViewer =true;
	
	// dead units should not be reported
	//if( !UnitState.IsAlive() )
	//{
	//	return;
	//}
	//
	//class'X2StatusEffects'.static.AddEffectMessageToTrack(BuildTrack, class'X2StatusEffects'.default.MarkedEffectLostString, VisualizeGameState.GetContext());
	//class'X2StatusEffects'.static.UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}

static function X2AbilityTemplate CoverAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityTarget_Single		SingleTarget;
	local X2AbilityCost_ActionPoints    ActionPointCost;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;
	local X2Condition_UnitProperty UnitPropertyCondition;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_Cover');

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);
	
	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.COVER_COOLDOWN;
	Cooldown.NumGlobalTurns = default.COVER_GLOBAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Defensive;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_aimcover";
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.ARMOR_ACTIVE_PRIORITY;

	Template.AbilityToHitCalc = default.DeadEye;

	SingleTarget = new class'X2AbilityTarget_Single';
	Template.AbilityTargetStyle = SingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	UnitPropertyCondition = new class'X2Condition_UnitProperty';
	UnitPropertyCondition.ExcludeDead = true;
	UnitPropertyCondition.ExcludeFriendlyToSource = false;
	UnitPropertyCondition.ExcludeHostileToSource = true;
	UnitPropertyCondition.ExcludeHostileToSource = true;
	UnitPropertyCondition.ExcludeCivilian = true;
	UnitPropertyCondition.FailOnNonUnits = true;
	Template.AbilityTargetConditions.AddItem(UnitPropertyCondition);
	
	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(2, false, true, false, eGameRule_PlayerTurnEnd);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Defense, default.COVER_DEFENSE);
	Template.AddTargetEffect(PersistentStatChangeEffect);

	Template.bShowActivation = true;
	Template.bSkipFireAction = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	return Template;
}

static function X2AbilityTemplate CreateEnergyShieldAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityCost_ActionPoints ActionPointCost;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;
	local X2Condition_UnitProperty UnitPropertyCondition;
	local X2AbilityTrigger_PlayerInput InputTrigger;
	local X2Effect_PersistentStatChange ShieldedEffect;
	local X2AbilityMultiTarget_Radius MultiTarget;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_EnergyShield');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_adventshieldbearer_energyshield";

	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.Hostility = eHostility_Defensive;

	// This ability is a free action
	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);
	
	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.ENERGY_SHIELD_COOLDOWN;
	Cooldown.NumGlobalTurns = default.ENERGY_SHIELD_GLOBAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	//Can't use while dead
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);

	// Add dead eye to guarantee
	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;

	// Multi target
	MultiTarget = new class'X2AbilityMultiTarget_Radius';
	MultiTarget.fTargetRadius = default.ENERGY_SHIELD_RANGE_METERS;
	MultiTarget.bIgnoreBlockingCover = true;
	Template.AbilityMultiTargetStyle = MultiTarget;

	InputTrigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(InputTrigger);

	// The Targets must be within the AOE, LOS, and friendly
	UnitPropertyCondition = new class'X2Condition_UnitProperty';
	UnitPropertyCondition.ExcludeDead = true;
	UnitPropertyCondition.ExcludeFriendlyToSource = false;
	UnitPropertyCondition.ExcludeHostileToSource = true;
	UnitPropertyCondition.ExcludeCivilian = true;
	UnitPropertyCondition.FailOnNonUnits = true;
	Template.AbilityMultiTargetConditions.AddItem(UnitPropertyCondition);

	// Friendlies in the radius receives a shield receives a shield
	ShieldedEffect = CreateShieldedEffect(Template.LocFriendlyName, Template.GetMyLongDescription(), default.ENERGY_SHIELD_HP);

	Template.AddShooterEffect(ShieldedEffect);
	Template.AddMultiTargetEffect(ShieldedEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = Shielded_BuildVisualization;
	//Template.ActivationSpeech = 
	//Template.CinescriptCameraType = "AdvShieldBearer_EnergyShieldArmor";
	
	return Template;
}

static function X2Effect_PersistentStatChange CreateShieldedEffect(string FriendlyName, string LongDescription, int ShieldHPAmount)
{
	local X2Effect_EnergyShield ShieldedEffect;

	ShieldedEffect = new class'X2Effect_EnergyShield';
	ShieldedEffect.BuildPersistentEffect(default.ENERGY_SHIELD_DURATION, false, true, , eGameRule_PlayerTurnEnd);
	ShieldedEffect.SetDisplayInfo(ePerkBuff_Bonus, FriendlyName, LongDescription, "img:///UILibrary_PerkIcons.UIPerk_adventshieldbearer_energyshield", true);
	ShieldedEffect.AddPersistentStatChange(eStat_ShieldHP, ShieldHPAmount);
	ShieldedEffect.EffectRemovedVisualizationFn = OnShieldRemoved_BuildVisualization;

	return ShieldedEffect;
}

simulated function OnShieldRemoved_BuildVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local X2Action_PlaySoundAndFlyOver SoundAndFlyOver;
	local X2Action_PlayEffect EffectAction;

	if (XGUnit(BuildTrack.TrackActor).IsAlive())
	{
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, VisualizeGameState.GetContext()));
		EffectAction.EffectName = "FX_AdvShieldBearer.P_AdvShieldBearer_Persistent_Shield";
		EffectAction.bStopEffect = true;

		SoundAndFlyOver = X2Action_PlaySoundAndFlyOver(class'X2Action_PlaySoundAndFlyOver'.static.AddToVisualizationTrack(BuildTrack, VisualizeGameState.GetContext()));
		SoundAndFlyOver.SetSoundAndFlyOverParameters(None, class'XLocalizedData'.default.ShieldRemovedMsg, '', eColor_Bad, , 0.75, true);
	}
}

simulated function Shielded_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference InteractingUnitRef;
	local VisualizationTrack EmptyTrack;
	local VisualizationTrack BuildTrack;
	local VisualizationTrack TargetBuildTrack;
	local X2Action_PlayAnimation PlayAnimationAction;
	local X2Action_PlayEffect EffectAction;
	local X2Action_TimedWait WaitAction;	
	//local X2VisualizerInterface TargetVisualizerInterface;
	local int i;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	//Configure the visualization track for the shooter
	//****************************************************************************************
	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	PlayAnimationAction = X2Action_PlayAnimation(class'X2Action_PlayAnimation'.static.AddToVisualizationTrack(BuildTrack, Context));
	PlayAnimationAction.Params.AnimName = 'HL_SignalPointA';//'HL_EnergyShieldA';

	WaitAction = X2Action_TimedWait(class'X2Action_TimedWait'.static.AddToVisualizationTrack(BuildTrack, Context));
	WaitAction.DelayTimeSec = 0.2;

	EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, Context));
	EffectAction.AttachToUnit = true;
	EffectAction.EffectName = "FX_AdvShieldBearer.P_AdvShieldBearer_Persistent_Shield";
	EffectAction.AttachToSocketName = default.ENERGY_SHIELD_ATTACHTOSOCKETNAME;
	EffectAction.AttachToSocketsArrayName = default.ENERGY_SHIELD_ATTACHTOSOCKETSARRAYNAME;
	
	for (i = 0; i < Context.InputContext.MultiTargets.Length; ++i)
	{
		InteractingUnitRef = Context.InputContext.MultiTargets[i];

		TargetBuildTrack = EmptyTrack;
		TargetBuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
		TargetBuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
		TargetBuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(TargetBuildTrack, Context));
		EffectAction.AttachToUnit = true;
		EffectAction.EffectName = "FX_AdvShieldBearer.P_AdvShieldBearer_Persistent_Shield";
		EffectAction.AttachToSocketName = default.ENERGY_SHIELD_ATTACHTOSOCKETNAME;
		EffectAction.AttachToSocketsArrayName = default.ENERGY_SHIELD_ATTACHTOSOCKETSARRAYNAME;

		//class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);
		//
		//for( j = 0; j < Context.ResultContext.MultiTargetEffectResults[i].Effects.Length; ++j )
		//{
		//	Context.ResultContext.MultiTargetEffectResults[i].Effects[j].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, Context.ResultContext.MultiTargetEffectResults[i].ApplyResults[j]);
		//}

		//TargetVisualizerInterface = X2VisualizerInterface(BuildTrack.TrackActor);
		//if( TargetVisualizerInterface != none )
		//{
		//	//Allow the visualizer to do any custom processing based on the new game state. For example, units will create a death action when they reach 0 HP.
		//	TargetVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
		//}

		OutVisualizationTracks.AddItem(TargetBuildTrack);
		
	}

	OutVisualizationTracks.AddItem(BuildTrack);
}

static function X2AbilityTemplate SecretMissionAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityTarget_Single		SingleTarget;
	local X2AbilityCharges                      Charges;
	local X2AbilityCost_ActionPoints    ActionPointCost;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;
	local X2Condition_UnitProperty UnitPropertyCondition;
	local X2Effect_ToHitModifier			Effect;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;
	local X2Effect_BonusWeaponDamage			DamageEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_SecretMission');

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);
	
	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = default.SECRET_MISSION_CHARGES;
	Template.AbilityCharges = Charges;

	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.SECRET_MISSION_COOLDOWN;
	Cooldown.NumGlobalTurns = default.SECRET_MISSION_GLOBAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Defensive;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_xenobiology_overlays";
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.ARMOR_ACTIVE_PRIORITY;

	Template.AbilityToHitCalc = default.DeadEye;

	SingleTarget = new class'X2AbilityTarget_Single';
	Template.AbilityTargetStyle = SingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	UnitPropertyCondition = new class'X2Condition_UnitProperty';
	UnitPropertyCondition.ExcludeDead = true;
	UnitPropertyCondition.ExcludeFriendlyToSource = false;
	UnitPropertyCondition.ExcludeHostileToSource = true;
	UnitPropertyCondition.ExcludeHostileToSource = true;
	UnitPropertyCondition.ExcludeCivilian = true;
	UnitPropertyCondition.FailOnNonUnits = true;
	Template.AbilityTargetConditions.AddItem(UnitPropertyCondition);
		
	Effect = new class'X2Effect_ToHitModifier';
	Effect.EffectName = 'Officer_SupplyCommand';
	Effect.DuplicateResponse = eDupe_Ignore;
	Effect.BuildPersistentEffect(1, false, false, false, eGameRule_PlayerTurnEnd);
	Effect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,false,,Template.AbilitySourceName);
	Effect.AddEffectHitModifier(eHit_Success,default.SECRET_MISSION_AIM, Template.LocFriendlyName);
	Template.AddTargetEffect(Effect);

	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(1, false, false, false, eGameRule_PlayerTurnEnd);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,false,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Dodge, default.SECRET_MISSION_DODGE);
	Template.AddTargetEffect(PersistentStatChangeEffect);

	DamageEffect = new class'X2Effect_BonusWeaponDamage';
	DamageEffect.BonusDmg = default.SECRET_MISSION_DAMAGE;
	DamageEffect.BuildPersistentEffect(1, false, false, false, eGameRule_PlayerTurnEnd);
	DamageEffect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
	Template.AddTargetEffect(DamageEffect);

	Template.bShowActivation = true;
	Template.bSkipFireAction = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	return Template;
}

static function X2AbilityTemplate SquadStealthAbility()
{
	local X2AbilityTemplate Template;
	local X2Effect_RangerStealth                StealthEffect;
	local X2AbilityCharges                      Charges;
	local X2AbilityMultiTarget_AllAllies		MultiTargetingStyle;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_SquadStealth');
	
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_stealth";
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_COLONEL_PRIORITY;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;

	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
	Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
	Template.AbilityCosts.AddItem(default.FreeActionCost);

	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = default.STEALTH_CHARGES;
	Template.AbilityCharges = Charges;

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AbilityShooterConditions.AddItem(new class'X2Condition_Stealth');

	StealthEffect = new class'X2Effect_RangerStealth';
	StealthEffect.BuildPersistentEffect(1, true, true, false, eGameRule_PlayerTurnEnd);
	StealthEffect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyHelpText(), Template.IconImage, true);
	StealthEffect.bRemoveWhenTargetConcealmentBroken = true;
	Template.AddMultiTargetEffect(StealthEffect);

	Template.AddMultiTargetEffect(class'X2Effect_Spotted'.static.CreateUnspottedEffect());

	Template.ActivationSpeech = 'ActivateConcealment';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	Template.bSkipFireAction = true;

	return Template;
}

static function X2AbilityTemplate AirRaidRequestAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityCost_ActionPoints ActionPointCost;
	local X2AbilityCooldown_LocalAndGlobal Cooldown;
	local X2AbilityMultiTarget_Radius MultiTarget;
	local X2Effect_MarkValidActivationTiles MarkTilesEffect;
	local X2AbilityTarget_Cursor CursorTarget;
	local X2Effect_DelayedAbilityActivation AirRaidDelayEffect;
	
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_AirRaidRequest');
	
	Template.AdditionalAbilities.AddItem('Officer_AirRaidAttack');
	Template.TwoTurnAttackAbility = 'Officer_AirRaidAttack';
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_gatekeeper_deathexplosion";

	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.bShowActivation = true;
	
	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Cooldown = new class'X2AbilityCooldown_LocalAndGlobal';
	Cooldown.iNumTurns = default.AIR_RAID_LOCAL_COOLDOWN;
	Cooldown.NumGlobalTurns = default.AIR_RAID_GLOBAL_COOLDOWN;
	Template.AbilityCooldown = Cooldown;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();
	
	MultiTarget = new class 'X2AbilityMultiTarget_Radius';
	MultiTarget.fTargetRadius =  default.AIR_RAID_IMPACT_RADIUS_METERS;
	MultiTarget.bIgnoreBlockingCover = true;
	Template.AbilityMultiTargetStyle = MultiTarget;
	
	MarkTilesEffect = new class'X2Effect_MarkValidActivationTiles';
	MarkTilesEffect.AbilityToMark = 'Officer_AirRaidAttack';
	MarkTilesEffect.OnlyUseTargetLocation = true;
	Template.AddShooterEffect(MarkTilesEffect);

	CursorTarget = new class'X2AbilityTarget_Cursor';
	CursorTarget.bRestrictToSquadsightRange = true;
	CursorTarget.FixedAbilityRange = default.AIR_RAID_TARGETING_AREA_RADIUS;
	Template.AbilityTargetStyle = CursorTarget;
	
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	AirRaidDelayEffect = new class 'X2Effect_DelayedAbilityActivation';
	AirRaidDelayEffect.BuildPersistentEffect(1, false, false, , eGameRule_PlayerTurnBegin);
	AirRaidDelayEffect.EffectName = default.AirRaidEffectName;
	AirRaidDelayEffect.TriggerEventName = default.AirRaidTriggerName;
	AirRaidDelayEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true, , Template.AbilitySourceName);
	Template.AddShooterEffect(AirRaidDelayEffect);
		
	Template.TargetingMethod = class'X2TargetingMethod_AirRaid';

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	Template.BuildVisualizationFn = AirRaidRequest_BuildVisualization;
	Template.BuildAffectedVisualizationSyncFn = AirRaidRequest_BuildAffectedVisualization;
	Template.CinescriptCameraType = "Codex_PsiBomb_Stage1";

	Template.DamagePreviewFn = AirRaidDamagePreview;

	//Template.ModifyNewContextFn = BlazingPinionsStage1_ModifyActivatedAbilityContext;
	//Template.BuildNewGameStateFn = BlazingPinionsStage1_BuildGameState;
	//Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	//Template.BuildVisualizationFn = BlazingPinionsStage1_BuildVisualization;
	//Template.BuildAppliedVisualizationSyncFn = BlazingPinionsStage1_BuildVisualizationSync;
	
	return Template;
}

function bool AirRaidDamagePreview(XComGameState_Ability AbilityState, StateObjectReference TargetRef, out WeaponDamageValue MinDamagePreview, out WeaponDamageValue MaxDamagePreview, out int AllowsShield)
{
	local XComGameState_Unit AbilityOwner;
	local StateObjectReference AirRaidSAttackRef;
	local XComGameState_Ability AirRaidAttackAbility;
	local XComGameStateHistory History;

	History = `XCOMHISTORY;
	AbilityOwner = XComGameState_Unit(History.GetGameStateForObjectID(AbilityState.OwnerStateObject.ObjectID));
	AirRaidSAttackRef = AbilityOwner.FindAbility('AirRaidAttack');
	AirRaidAttackAbility = XComGameState_Ability(History.GetGameStateForObjectID(AirRaidSAttackRef.ObjectID));
	if( AirRaidAttackAbility == none )
	{
		`RedScreenOnce("Unit has AirRaidRequest but is missing AirRaidAttack. No es Bueno. -dslonneger @gameplay");
	}
	else
	{
		AirRaidAttackAbility.GetDamagePreview(TargetRef, MinDamagePreview, MaxDamagePreview, AllowsShield);
	}
	return true;
}

simulated function AirRaidRequest_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability Context;
	local StateObjectReference InteractingUnitRef;
	local X2VisualizerInterface Visualizer;
	local VisualizationTrack AirRaidBuildTrack;
	local X2Action_PlayEffect EffectAction;
	local XComGameState_Unit AirRaidUnit;
	local XComWorldData World;
	local vector TargetLocation;
	local TTile TargetTile;
	local X2Action_TimedWait WaitAction;
	local X2Action_PlayAnimation PlayAnimation;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());

	//Configure the visualization track for the shooter
	//****************************************************************************************
	InteractingUnitRef = Context.InputContext.SourceObject;
	AirRaidBuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	AirRaidBuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	AirRaidBuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	AirRaidUnit = XComGameState_Unit(AirRaidBuildTrack.StateObject_NewState);

	if( AirRaidUnit != none )
	{
		World = `XWORLD;

		// Exit cover
		class'X2Action_ExitCover'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context);

		class'X2Action_Fire_OpenUnfinishedAnim'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context);

		PlayAnimation = X2Action_PlayAnimation(class'X2Action_PlayAnimation'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context));
		PlayAnimation.Params.AnimName = 'HL_SignalPointA';

		// Wait to time the start of the warning FX
		WaitAction = X2Action_TimedWait(class'X2Action_TimedWait'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context));
		WaitAction.DelayTimeSec = 0.5;

		// Display the Warning FX (convert to tile and back to vector because stage 2 is at the GetPositionFromTileCoordinates coord
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context));
		EffectAction.EffectName = "FX_Advent_Troop_Transport.P_Transport_Arrival_Warning";

		TargetLocation = Context.InputContext.TargetLocations[0];
		TargetTile = World.GetTileCoordinatesFromPosition(TargetLocation);

		EffectAction.EffectLocation = World.GetPositionFromTileCoordinates(TargetTile);
		
		Visualizer = X2VisualizerInterface(AirRaidBuildTrack.TrackActor);
		if( Visualizer != none )
		{
			Visualizer.BuildAbilityEffectsVisualization(VisualizeGameState, AirRaidBuildTrack);
		}

		class'X2Action_EnterCover'.static.AddToVisualizationTrack(AirRaidBuildTrack, Context);

		OutVisualizationTracks.AddItem(AirRaidBuildTrack);
	}
}

simulated function AirRaidRequest_BuildAffectedVisualization(name EffectName, XComGameState VisualizeGameState, out VisualizationTrack BuildTrack )
{
	local XComGameStateContext_Ability Context;
	local X2Action_PlayEffect EffectAction;
	//local X2Action_StartStopSound SoundAction;
	local XComGameState_Unit AirRaidUnit;
	local XComWorldData World;
	local vector TargetLocation;
	local TTile TargetTile;
	
	if( !`XENGINE.IsMultiplayerGame() && EffectName == default.AirRaidEffectName )
	{
		Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
		AirRaidUnit = XComGameState_Unit(BuildTrack.StateObject_NewState);

		if( (Context == none) || (AirRaidUnit == none) )
		{
			return;
		}

		World = `XWORLD;
		// Display the Warning FX (convert to tile and back to vector because stage 2 is at the GetPositionFromTileCoordinates coord
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(BuildTrack, Context));
		EffectAction.EffectName = "FX_Advent_Troop_Transport.P_Transport_Arrival_Warning";

		TargetLocation = Context.InputContext.TargetLocations[0];
		TargetTile = World.GetTileCoordinatesFromPosition(TargetLocation);

		EffectAction.EffectLocation = World.GetPositionFromTileCoordinates(TargetTile);
	}
}

static function X2AbilityTemplate AirRaidAttackAbility()
{
	local X2AbilityTemplate Template;
	local X2AbilityMultiTarget_Radius RadiusMultiTarget;
	local X2Condition_UnitProperty LivingTargetCondition;
	local X2AbilityTrigger_EventListener DelayedEventListener;
	local X2Effect_ApplyWeaponDamage DamageEffect;
	local X2Effect_PerkAttachForFX FXEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_AirRaidAttack');

	Template.bDontDisplayInAbilitySummary = true;
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.AbilitySourceName = 'eAbilitySource_Psionic';

	Template.AbilityToHitCalc = default.DeadEye;

	LivingTargetCondition = new class'X2Condition_UnitProperty';
	LivingTargetCondition.ExcludeFriendlyToSource = false;
	LivingTargetCondition.ExcludeHostileToSource = false;
	LivingTargetCondition.ExcludeAlive = false;
	LivingTargetCondition.ExcludeDead = true;
	LivingTargetCondition.FailOnNonUnits = true;
	Template.AbilityMultiTargetConditions.AddItem(LivingTargetCondition);

	RadiusMultiTarget = new class'X2AbilityMultiTarget_Radius';
	RadiusMultiTarget.fTargetRadius = default.AIR_RAID_IMPACT_RADIUS_METERS;
	RadiusMultiTarget.bIgnoreBlockingCover = true;
	Template.AbilityMultiTargetStyle = RadiusMultiTarget;

	//AIR_RAID_TARGETING_AREA_RADIUS;
	//AIR_RAID_DAMAGE_AMOUNT;

	// TODO: This doesn't actually target self but needs an AbilityTargetStyle
	Template.AbilityTargetStyle = default.SelfTarget;

	// This ability fires when the event DelayedExecuteRemoved fires on this unit
	DelayedEventListener = new class'X2AbilityTrigger_EventListener';
	DelayedEventListener.ListenerData.Deferral = ELD_OnStateSubmitted;
	DelayedEventListener.ListenerData.EventID =  default.AirRaidTriggerName;
	DelayedEventListener.ListenerData.Filter = eFilter_Unit;
	DelayedEventListener.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_ValidAbilityLocation;
	Template.AbilityTriggers.AddItem(DelayedEventListener);

	// This effect is here to attach perk FX to
	FXEffect = new class'X2Effect_PerkAttachForFX';
	Template.AddShooterEffect(FXEffect);

	DamageEffect = new class'X2Effect_ApplyWeaponDamage';
	DamageEffect.EnvironmentalDamageAmount = default.AIR_RAID_DAMAGE_AMOUNT;
	DamageEffect.EffectDamageValue = default.AIR_RAID_WEAPON_DAMAGE;
	DamageEffect.EffectDamageValue.DamageType = 'Explosion';
	DamageEffect.bExplosiveDamage = true;
	Template.AddMultiTargetEffect(DamageEffect);
	Template.AddMultiTargetEffect(new class'X2Effect_ApplyFireToWorld');

	Template.bSkipFireAction = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	Template.BuildVisualizationFn = AirRaidAttack_BuildVisualization;
	Template.CinescriptCameraType = "Codex_PsiBomb_Stage2";

	return Template;
}

simulated function AirRaidAttack_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference InteractingUnitRef;
	local X2AbilityTemplate AbilityTemplate;
	local VisualizationTrack EmptyTrack;
	local VisualizationTrack CyberusBuildTrack, BuildTrack;
	local int i, j;
	local X2VisualizerInterface TargetVisualizerInterface;
	local XComGameState_EnvironmentDamage EnvironmentDamageEvent;
	local XComGameState_WorldEffectTileData WorldDataUpdate;
	local XComGameState_InteractiveObject InteractiveObject;
	local X2Action_PlayEffect EffectAction;
	local X2Action_StartStopSound SoundAction;
	local XComGameState_Unit CyberusUnit;
	local X2Action_TimedInterTrackMessageAllMultiTargets MultiTargetMessageAction;
	local X2Action_TimedWait WaitAction;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	AbilityTemplate = class'XComGameState_Ability'.static.GetMyTemplateManager().FindAbilityTemplate(Context.InputContext.AbilityTemplateName);

	//****************************************************************************************
	//Configure the visualization track for the source
	//****************************************************************************************
	CyberusBuildTrack = EmptyTrack;
	History.GetCurrentAndPreviousGameStatesForObjectID(InteractingUnitRef.ObjectID,
													   CyberusBuildTrack.StateObject_OldState, CyberusBuildTrack.StateObject_NewState,
													   eReturnType_Reference,
													   VisualizeGameState.HistoryIndex);
	CyberusBuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	CyberusUnit = XComGameState_Unit(CyberusBuildTrack.StateObject_OldState);

	if( CyberusUnit != none )
	{
		// Stop the Loop audio
		SoundAction = X2Action_StartStopSound(class'X2Action_StartStopSound'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		SoundAction.Sound = new class'SoundCue';
		SoundAction.Sound.AkEventOverride = AkEvent'SoundX2CyberusFX.Stop_CodexPsiBombLoop';
		SoundAction.iAssociatedGameStateObjectId = InteractingUnitRef.ObjectID;
		SoundAction.bIsPositional = true;
		SoundAction.bStopPersistentSound = true;

		// Stop the Warning FX
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		EffectAction.EffectName = "FX_Advent_Troop_Transport.P_Transport_Arrival_Warning";
		EffectAction.EffectLocation = Context.InputContext.TargetLocations[0];
		EffectAction.bStopEffect = true;

		// Play the Collapsing audio
		SoundAction = X2Action_StartStopSound(class'X2Action_StartStopSound'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		SoundAction.Sound = new class'SoundCue';
		SoundAction.Sound.AkEventOverride = AkEvent'SoundX2ArchonFX.Blazing_Pinoins_Incoming_Wide';
		SoundAction.bIsPositional = true;
		SoundAction.vWorldPosition = Context.InputContext.TargetLocations[0];

		// Play the Collapse FX
		//EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		//EffectAction.EffectName = "FX_Archon.P_Blazing_Pinions_Explosion";
		//EffectAction.EffectLocation = Context.InputContext.TargetLocations[0];
		//EffectAction.bWaitForCompletion = false;
		//EffectAction.bWaitForCameraCompletion = false;

		// Wait to time the start of the explosion FX
		WaitAction = X2Action_TimedWait(class'X2Action_TimedWait'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		WaitAction.DelayTimeSec = 1.5;//default.PSI_BOMB_STAGE2_START_EXPLOSION_FX_SEC;

		// Play the Explosion audio
		SoundAction = X2Action_StartStopSound(class'X2Action_StartStopSound'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		SoundAction.Sound = new class'SoundCue';
		SoundAction.Sound.AkEventOverride = AkEvent'SoundX2GrenadeFX.GrenadeExplosion';
		SoundAction.bIsPositional = true;
		SoundAction.vWorldPosition = Context.InputContext.TargetLocations[0];

		// Play the Explosion FX
		EffectAction = X2Action_PlayEffect(class'X2Action_PlayEffect'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		//EffectAction.EffectName = "FX_Archon.P_Blazing_Pinions_Explosion";
		EffectAction.EffectName = "FX_Explosion_Grenade_Launcher.P_Grenade_Launcher_Explosion";

		EffectAction.EffectLocation = Context.InputContext.TargetLocations[0];

		// Notify multi targets of explosion
		MultiTargetMessageAction = X2Action_TimedInterTrackMessageAllMultiTargets(class'X2Action_TimedInterTrackMessageAllMultiTargets'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
		MultiTargetMessageAction.SendMessagesAfterSec = 0.2;//default.PSI_BOMB_STAGE2_NOTIFY_TARGETS_SEC;
	}
	//****************************************************************************************

	//****************************************************************************************
	//Configure the visualization track for the targets
	//****************************************************************************************
	for (i = 0; i < Context.InputContext.MultiTargets.Length; ++i)
	{
		InteractingUnitRef = Context.InputContext.MultiTargets[i];

		if( InteractingUnitRef == CyberusUnit.GetReference() )
		{
			BuildTrack = CyberusBuildTrack;

			//WaitAction = X2Action_TimedWait(class'X2Action_TimedWait'.static.AddToVisualizationTrack(CyberusBuildTrack, Context));
			//WaitAction.DelayTimeSec = 1;//default.PSI_BOMB_STAGE2_NOTIFY_TARGETS_SEC;
		}
		else
		{
			BuildTrack = EmptyTrack;
			BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
			BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
			BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

			class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);
		}

		for( j = 0; j < Context.ResultContext.MultiTargetEffectResults[i].Effects.Length; ++j )
		{
			Context.ResultContext.MultiTargetEffectResults[i].Effects[j].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, Context.ResultContext.MultiTargetEffectResults[i].ApplyResults[j]);
		}

		TargetVisualizerInterface = X2VisualizerInterface(BuildTrack.TrackActor);
		if( TargetVisualizerInterface != none )
		{
			//Allow the visualizer to do any custom processing based on the new game state. For example, units will create a death action when they reach 0 HP.
			TargetVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
		}

		if (BuildTrack.TrackActions.Length > 0)
		{
			OutVisualizationTracks.AddItem(BuildTrack);
		}
	}

	OutVisualizationTracks.AddItem(CyberusBuildTrack);

	//****************************************************************************************
	//Configure the visualization tracks for the environment
	//****************************************************************************************
	foreach VisualizeGameState.IterateByClassType(class'XComGameState_EnvironmentDamage', EnvironmentDamageEvent)
	{
		BuildTrack = EmptyTrack;
		BuildTrack.TrackActor = none;
		BuildTrack.StateObject_NewState = EnvironmentDamageEvent;
		BuildTrack.StateObject_OldState = EnvironmentDamageEvent;

		//Wait until signaled by the shooter that the projectiles are hitting
		class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);

		for( i = 0; i < AbilityTemplate.AbilityMultiTargetEffects.Length; ++i )
		{
			AbilityTemplate.AbilityMultiTargetEffects[i].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, 'AA_Success');	
		}

		OutVisualizationTracks.AddItem(BuildTrack);
	}

	foreach VisualizeGameState.IterateByClassType(class'XComGameState_WorldEffectTileData', WorldDataUpdate)
	{
		BuildTrack = EmptyTrack;
		BuildTrack.TrackActor = none;
		BuildTrack.StateObject_NewState = WorldDataUpdate;
		BuildTrack.StateObject_OldState = WorldDataUpdate;

		//Wait until signaled by the shooter that the projectiles are hitting
		class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);

		for( i = 0; i < AbilityTemplate.AbilityMultiTargetEffects.Length; ++i )
		{
			AbilityTemplate.AbilityMultiTargetEffects[i].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, 'AA_Success');	
		}

		OutVisualizationTracks.AddItem(BuildTrack);
	}
	//****************************************************************************************

	//Process any interactions with interactive objects
	foreach VisualizeGameState.IterateByClassType(class'XComGameState_InteractiveObject', InteractiveObject)
	{
		// Add any doors that need to listen for notification
		if( InteractiveObject.IsDoor() && InteractiveObject.HasDestroyAnim() && InteractiveObject.InteractionCount % 2 != 0 ) //Is this a closed door?
		{
			BuildTrack = EmptyTrack;
			//Don't necessarily have a previous state, so just use the one we know about
			BuildTrack.StateObject_OldState = InteractiveObject;
			BuildTrack.StateObject_NewState = InteractiveObject;
			BuildTrack.TrackActor = History.GetVisualizer(InteractiveObject.ObjectID);
			class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);
			class'X2Action_BreakInteractActor'.static.AddToVisualizationTrack(BuildTrack, Context);

			OutVisualizationTracks.AddItem(BuildTrack);
		}
	}
}


static function X2AbilityTemplate AlertCommandAbility()
{
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies	MultiTargetingStyle;
	local X2Effect_PersistentStatChange     StatChangeEffect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_AlertCommand');
	Template.IconImage =  "img:///UILibrary_PerkIcons.UIPerk_battlescanner";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	StatChangeEffect = new class'X2Effect_PersistentStatChange';
	StatChangeEffect.BuildPersistentEffect(1, true,true, false);
	StatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	StatChangeEffect.AddPersistentStatChange(eStat_SightRadius, default.ALERT_COMMAND_SIGHT);
	//Template.AddTargetEffect(StatChangeEffect);
	Template.AddMultiTargetEffect(StatChangeEffect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

static function X2AbilityTemplate EncourageCommandAbility()
{
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies	MultiTargetingStyle;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_EncourageCommand');
	Template.IconImage =  "img:///UILibrary_PerkIcons.UIPerk_beserk";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(1, true, true, false);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Will, default.ENCOURAGE_COMMAND_BONUS_WILL);
	//Template.AddTargetEffect(PersistentStatChangeEffect);
	Template.AddMultiTargetEffect(PersistentStatChangeEffect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

static function X2AbilityTemplate DefenseCommandAbility()
{
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies		MultiTargetingStyle;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;
	//local X2Effect_DefenseCommand				CommandEffect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_DefenseCommand');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_civiliancover";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(1, true, true, false);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_ArmorChance, default.DEFENSE_COMMAND_CHANCE);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_ArmorMitigation, default.DEFENSE_COMMAND_AMOUNT);
	Template.AddMultiTargetEffect(PersistentStatChangeEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	return Template;
}

static function X2AbilityTemplate BlitzCommandAbility()
{
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies	MultiTargetingStyle;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_BlitzCommand');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_adventstunlancer_charge";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);
	
	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(1, true, true, false);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Mobility, default.BLITZ_COMMAND_MOVE);
	//Template.AddTargetEffect(PersistentStatChangeEffect);
	Template.AddMultiTargetEffect(PersistentStatChangeEffect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	return Template;
}

static function X2AbilityTemplate FireCommandAbility()
{	
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies		MultiTargetingStyle;
	local X2AbilityTrigger						Trigger;
	local X2Effect_ToHitModifier                Effect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_FireCommand');

	Template.IconImage =  "img:///UILibrary_PerkIcons.UIPerk_aggression";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	
	Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	Template.AbilityTriggers.AddItem(Trigger);

	Effect = new class'X2Effect_ToHitModifier';
	Effect.EffectName = 'Officer_FireCommand';
	Effect.DuplicateResponse = eDupe_Ignore;
	Effect.BuildPersistentEffect(1, true, false);
	Effect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	//Effect.AddEffectHitModifier(eHit_Success, default.SHADOWSTRIKE_AIM, Template.LocFriendlyName);
	Effect.AddEffectHitModifier(eHit_Crit, default.FIRE_COMMAND_CIRT, Template.LocFriendlyName); // default.FIRE_COMMAND_CIRT
	//Template.AddTargetEffect(Effect);
	Template.AddMultiTargetEffect(Effect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

static function X2AbilityTemplate TacticalCommandAbility()
{
	local X2AbilityTemplate						Template;
	local X2AbilityMultiTarget_AllAllies	MultiTargetingStyle;
	local X2Effect_ToHitModifier                Effect;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Officer_TacticalCommand');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_gunslinger";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.bUniqueSource = true;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	MultiTargetingStyle = new class'X2AbilityMultiTarget_AllAllies';
	MultiTargetingStyle.bAllowSameTarget = true;
	Template.AbilityMultiTargetStyle = MultiTargetingStyle;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	Effect = new class'X2Effect_ToHitModifier';
	Effect.EffectName = 'Officer_SupplyCommand';
	Effect.DuplicateResponse = eDupe_Ignore;
	Effect.BuildPersistentEffect(1, true, false);
	Effect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,false,,Template.AbilitySourceName);
	Effect.AddEffectHitModifier(eHit_Success,default.TACTICAL_COMMAND_AIM, Template.LocFriendlyName);
	Template.AddMultiTargetEffect(Effect);

	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.BuildPersistentEffect(1, true, true, false);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Dodge, default.TACTICAL_COMMAND_DODGE);
	//Template.AddTargetEffect(PersistentStatChangeEffect);
	Template.AddMultiTargetEffect(PersistentStatChangeEffect);
	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

defaultproperties
{
	AirRaidEffectName="AirRaidEffect"
	AirRaidTriggerName="AirRaidTrigger"
}
